#include "prjlibs-c/standards.h"
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/vt.h>
#include <sys/sysmacros.h>

#include "skalibs/include/stddjb.h"
#include "prjlibs-c/constants.h"
#include "fdtools.h"

char const* PROG="vc-get";
#define dev_tty "/dev/tty"

int main(int argc, char** argv) {
  if (argc<3)
    strerr_die3x(100, usage, PROG, " {fd|/dev/ttyN} program [arg ...]");

  {
    int fd, scan;
    char buf[sizeof dev_tty+(sizeof (int)*CHAR_BIT+2)/3]=dev_tty;

    fd=scan_or_open(&scan, argv[1], O_RDONLY, 0);
    if (fd<0) {
      if (scan) strerr_die3x(100, PROG, ": descriptor malformed: ", argv[1]);
      else strerr_die5sys(errstat, PROG, err_unable, err_open, argv[1], ": ");
    }

    {
      int vtnum=-1;
      char* const bufnum=buf+sizeof dev_tty-1;
      errno=0;
      if (ioctl(fd, VT_OPENQRY, &vtnum)<0 || vtnum==-1)
        strerr_die3sys(errstat, PROG, err_unable, "find available console: ");
      bufnum[fmt_ulong(bufnum, vtnum)]='\0';
      if (pathexec_env("TTY", buf)==0) goto env_err;
    }
    close(fd);

    fd=open(buf, O_RDWR|O_NOCTTY);
    if (fd<0)
      strerr_die5sys(errstat, PROG, err_unable, err_open, buf, ": ");
    buf[fmt_ulong(buf, fd)]='\0';
    if (pathexec_env("VCFD", buf)==0) goto env_err;

    {
      struct stat statbuf;
      if (fstat(fd, &statbuf)!=0)
        strerr_die4sys(errstat, PROG, err_unable, err_stat, "descriptor: ");
      buf[fmt_ulong(buf, minor(statbuf.st_rdev))]='\0';
    }
    if (pathexec_env("VCNUM", buf)==0) goto env_err;
  }

  pathexec((char const**)argv+2);
  strerr_die5sys((errno==ENOENT? 127: errstat), PROG, err_unable, err_stat,
                 argv[2], ": ");
  env_err:
  strerr_die2sys(111, PROG, ": unable to set environment: ");
}
